import { __decorate } from 'tslib';
import { Injectable, SecurityContext, Input, Component, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { DomSanitizer } from '@angular/platform-browser';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { takeUntil } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/** Notification Class */
class Notification {
    /**
     * Constructor For Notification
     *
     * @param message message to be shown
     * @param style bootstrap background colors
     * @param duration timeout for notification
     */
    constructor(message, style, duration) {
        this.id = '_' + Math.random().toString(36).substr(2, 9); // generate a random number
        this.message = message;
        this.style = style || 'info';
        this.duration = duration;
    }
}

/** Notification Service, recieves notifications from user and interacts with components */
let NgxNotifierService = class NgxNotifierService {
    constructor() {
        /** notification which can be subscribed on new messages */
        this.notification = new Subject();
        /** clear all toast notifications */
        this.clearToasts = new Subject();
        /** clear last toast notification */
        this.clearLastToast = new Subject();
    }
    /** pushes a new notification */
    createToast(message, style, duration) {
        this.notification.next(new Notification(message, style, duration));
    }
    /** clear all toast notifications */
    clear() {
        this.clearToasts.next();
    }
    /** clear last toast notification */
    clearLast() {
        this.clearLastToast.next();
    }
};
NgxNotifierService = __decorate([
    Injectable()
], NgxNotifierService);

/**
 * Notifier compoent, which holds all the notifications can be accessed via `ngx-notifier` selector
 */
let NgxNotifierComponent = class NgxNotifierComponent {
    /**
     * NgxNotifierComponent Constructor
     *
     * @param ngxNotifierService subscribe to get values form notifier service
     */
    constructor(ngxNotifierService, domSanitizer) {
        this.ngxNotifierService = ngxNotifierService;
        this.domSanitizer = domSanitizer;
        this.componentDestroyed$ = new Subject();
        /** whether to allow duplicate messages or not */
        this.allowDuplicates = true;
        /** allow HTML in notification */
        this.allowHTML = false;
        /** default duration for dismissing notifications (60s/1minute) */
        this.duration = 60000;
        /** weather to enable or disable animations */
        this.disableAnimations = false;
        /** click to dismiss a notification */
        this.dismissOnClick = false;
        /** whether to insert on top or at bottom */
        this.insertOnTop = true;
        /** Maximum number of notifications to keep */
        this.max = 5;
        /** array of notifications */
        this.notifications = [];
        this.ngxNotifierService.notification.pipe(takeUntil(this.componentDestroyed$)).subscribe((notification) => { this.updateNotifications(notification); });
        this.ngxNotifierService.clearToasts.pipe(takeUntil(this.componentDestroyed$)).subscribe(() => { this.notifications = []; });
        this.ngxNotifierService.clearLastToast.pipe(takeUntil(this.componentDestroyed$)).subscribe(() => { this.clearLastToast(); });
    }
    /**
     * updates notification into the array i.e., Which is the display
     *
     * @param notification notification element
     */
    updateNotifications(notification) {
        // checks whether the message is alrady present in notifications
        const index = this.notifications.map((e) => e.message).indexOf(notification.message);
        if (!this.allowDuplicates) {
            if (index !== -1) {
                return;
            }
        }
        // save the last inserted Id
        this.lastInsertedNotificationId = notification.id;
        // sanitize html if enableHTML is set to true
        let sanitizedMessage;
        if (notification.message && this.allowHTML) {
            sanitizedMessage = this.domSanitizer.sanitize(SecurityContext.HTML, notification.message);
        }
        // set sanitized output to notification message
        notification.message = sanitizedMessage || notification.message;
        // insert notification in the first position of the array
        if (this.insertOnTop) {
            this.notifications.unshift(notification);
        }
        else {
            this.notifications.push(notification);
        }
        /**
         * remove the last inserted element if max has
         * pop or shift based on `insertOnTop`
         */
        if (this.notifications.length > this.max) {
            if (this.insertOnTop) {
                this.notifications.pop();
            }
            else {
                this.notifications.shift();
            }
        }
        // clear notification in given time
        setTimeout(() => {
            this.notifications.splice(index, 1);
        }, notification.duration || this.duration);
        return;
    }
    /**
     * remove the element from the array based on index
     *
     * @param index position of the element
     */
    removeNotification(index) {
        if (index !== undefined || index !== null) {
            this.notifications.splice(index, 1);
        }
        return;
    }
    /**
     * click event when toast is cleared
     *
     * @param index position of the element
     */
    onToastClick(index) {
        if (this.dismissOnClick) {
            this.removeNotification(index);
        }
        return;
    }
    /** clear last inserted toast notification */
    clearLastToast() {
        const index = this.notifications.map((e) => e.id).indexOf(this.lastInsertedNotificationId);
        if (this.notifications.length !== 0 && index !== -1) {
            this.notifications.splice(index, 1);
        }
        return;
    }
    /** stop subscription when component is destroyed */
    ngOnDestroy() {
        this.componentDestroyed$.next();
        this.componentDestroyed$.complete();
    }
};
NgxNotifierComponent.ctorParameters = () => [
    { type: NgxNotifierService },
    { type: DomSanitizer }
];
__decorate([
    Input()
], NgxNotifierComponent.prototype, "allowDuplicates", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "allowHTML", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "className", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "duration", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "disableAnimations", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "dismissOnClick", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "insertOnTop", void 0);
__decorate([
    Input()
], NgxNotifierComponent.prototype, "max", void 0);
NgxNotifierComponent = __decorate([
    Component({
        selector: 'ngx-notifier',
        template: "<div class=\"ngx-notifier\" *ngIf=\"notifications?.length !== 0\" [ngClass]=\"className\" [@animateToasts] [@.disabled]=\"disableAnimations\">\n  <div [ngClass]=\"{'pointer': dismissOnClick}\">\n    <div class=\"ngx-n-notification alert\" role=\"alert\" *ngFor=\"let notification of notifications; let i = index\"\n      [ngClass]=\"'alert-'+ notification.style\" (click)=\"onToastClick(i)\" [@animateToasts]>\n      <button type=\"button\" class=\"close\" aria-label=\"Close\" (click)=\"removeNotification(i)\">\n        <span aria-hidden=\"true\">&times;</span>\n      </button>\n      <p *ngIf=\"allowHTML\" [innerHTML]=\"notification.message\"> </p>\n      <p *ngIf=\"!allowHTML\">{{ notification.message }} </p>\n    </div>\n  </div>\n</div>\n",
        animations: [
            trigger('animateToasts', [
                state('void', style({ opacity: 0 })),
                transition(':enter, :leave', [
                    animate('0.3s ease')
                ])
            ])
        ],
        styles: [".chevron::before{border-style:solid;border-width:.15rem .15rem 0 0;content:\"\";display:inline-block;height:.45em;position:relative;transform:rotate(-45deg);width:.45em}.chevron.right::before{left:0;transform:rotate(45deg)}.chevron.bottom::before{top:0;transform:rotate(135deg)}.chevron.left::before{left:.25em;transform:rotate(-135deg)}.ngx-notifier{position:fixed;min-width:25rem;max-width:25rem;right:1rem;bottom:1rem}.ngx-notifier .pointer{cursor:pointer}.ngx-notifier .ngx-n-notification{margin-top:.5rem;border-radius:4px;padding:.5rem}.ngx-notifier .ngx-n-notification p{margin:0}.ngx-notifier .ngx-n-notification .close{font-size:1.2rem}.ngx-notifier .ngx-n-notification .close:focus{outline:0}@media only screen and (max-width:420px){.ngx-notifier{min-width:20rem;max-width:20rem}}"]
    })
], NgxNotifierComponent);

let NgxNotifierModule = class NgxNotifierModule {
};
NgxNotifierModule = __decorate([
    NgModule({
        imports: [CommonModule],
        declarations: [NgxNotifierComponent],
        providers: [NgxNotifierService],
        exports: [NgxNotifierComponent]
    })
], NgxNotifierModule);

/*
 * Public API Surface of ngx-notifier
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxNotifierComponent, NgxNotifierModule, NgxNotifierService };
//# sourceMappingURL=ngx-notifier.js.map
